@@ -8,6 +8,8 @@ func HasFullscreenRenderer() bool {
 	return false
 }
 
+var DefaultBorderShape BorderShape = BorderRounded
+
 func (a Attr) Merge(b Attr) Attr {
 	return a | b
 }
@@ -32,6 +34,7 @@ func (r *FullscreenRenderer) Resize(maxHeightFunc func(int) int) {}
 func (r *FullscreenRenderer) Pause(bool)                         {}
 func (r *FullscreenRenderer) Resume(bool, bool)                  {}
 func (r *FullscreenRenderer) Clear()                             {}
+func (r *FullscreenRenderer) NeedScrollbarRedraw() bool          { return false }
 func (r *FullscreenRenderer) Refresh()                           {}
 func (r *FullscreenRenderer) Close()                             {}
 
